### Explanations on the implementation for obtaining the deterministic cell-type frequencies 

`Function module.celltypefrequencies(u,n,s,Type)`

We use the time-derivates of cell-type abundances $\dot N_i$ given by Eq.(C.5-7)
\begin{equation}
\dot N_i 
= \sum \limits_{k=0}^n \left(\Lambda_k N_k \left\{p^k_i - \frac{N_i}{N-1}\right\}+ \Lambda_i N_i \left\{p^i_i - \frac{N-2}{N-1}\right\}\right)+(-\delta_{0,i}+\delta_{1,i})\Lambda_0 N_0 n u 
\end{equation}
and derive the time-derivatives of the frequencies $\dot x_i$ for an infinite population ($N-1\approx N$)
\begin{equation}
\dot x_i 
= \sum \limits_{k=0}^n \left(\Lambda_k x_k \left\{p^k_i - x_i\right\}+ \Lambda_i x_i \left\{p^i_i -1\right\}\right)
+(-\delta_{0,i}+\delta_{1,i})\Lambda_0 x_0 n u 
\end{equation}

We rewrite this as a matrix equation
\begin{equation}
\mathbf{\dot x}=(\mathbf{M}_1-\mathbf{M}_2(\mathbf{x}))\cdot\mathbf{x}+\mathbf{e}_{01}\Lambda_0 x_0 nu \equiv \mathbf f( \mathbf x )
\end{equation}
where
we defined matrices by
\begin{align}
\mathbf{M}_1\equiv\left(\Lambda_k p^k_i-\Lambda_i(1-p^i_i)\delta_{i,k}\right)_{ik}
\end{align}
\begin{align}
\mathbf{M}_2\equiv\left(\Lambda_k x_i\right)_{ik}=\mathbf x \otimes \Lambda
\end{align}
and the vectors
\begin{align}
\mathbf{e}_{01}\equiv(-1,1,0,\dots,0)
\end{align}
\begin{align}
\mathbf{\Lambda}\equiv(\Lambda_i)_i
\end{align}

In the function `celltypefrequencies(u,n,s,Type)`, we implemented the integration for the IVP defined by $\{\dot x_i\}$ where $i\in\{0,\dots,n\}$ with initial values $x_i(t=0)=\delta_{i,0}$.

The system is integrated until time derivative of the frequencies $x_i\le10^{15}$ for all $i$ to find the equilibrium, i.e. the cell-type frequencies in the SGV.


