import numpy as np
from scipy.special import binom
from numpy import eye as e

def sgvstochastic(Type, n, sigma, N, u, tmax, seed):
  # define dictionary for array of replication rates Lambda
  lDic={
      "Dom" : np.array([1.-(i!=0)*sigma for i in range(n+1)]),
      "Rec" : np.array([1.-(i==n)*sigma for i in range(n+1)]),
      "Int" : np.array([1.-i/n*sigma for i in range(n+1)]),
      "GDE" : np.array([1.-i*sigma for i in range(n+1)]),
  }
  l=lDic[Type] # define array for replication rates Lambda

  # define vector of cell-type abundances
  NVec=np.zeros(n+1)
  NVec[0]=N

  # define function for p^i_{j1,j2}
  def p(i,j1,j2):
      return binom(n-i,j1+j2-2*i)*\
              u**(j1+j2-2*i)*\
              (1.-u)**(n-i-(j1+j2-2*i))*\
              binom(j1+j2,j1)*\
              binom(2*n-(j1+j2),n-j1)/\
              binom(2*n,n)

  # define array for the prefactor Lambda_i/(N-1)*p^i_{j1,j2}
  pre=np.fromfunction(lambda i,j1,j2: l[i]/(N-1)*p(i,j1,j2),\
                 (n+1, n+1,n+1),dtype=int)
#

  # define function for reaction vector for parameters i,k,j1,j2
  def e1(i,j1,j2,k):
      return -e(n+1)[i]-e(n+1)[k]+e(n+1)[j1]+e(n+1)[j2]
  # define array for all reactions

  # define array of reactionsindices and filter for reactions that change the
  # system (nonzeroreactions)
  rn=range(n+1)
  reactionindices=np.array([np.array([i,j1,j2,k]) for i in rn for j1 in rn for j2 in rn for k in rn ])
  nonzeroreactions=([np.any(p(*reactionindices[ind][[0,1,2]])*e1(*(reactionindices[ind]))!=0) for ind in range(len(reactionindices))])
  reactionindices=reactionindices[nonzeroreactions]
  reactions=np.array([e1(i,j1,j2,k) for (i,j1,j2,k) in reactionindices])

  # gillespie algorithm
  np.random.seed(seed=seed)
  t=0.
  while True:
      if t==0.: # for first step: calculate all rates
          rates=np.array([NVec[i]*NVec[k]*pre[i,j1,j2]\
                  for (i,j1,j2,k) in reactionindices])
      else: # for all other steps: calculate only changed
            # rates
          reactionindicesOfreactionsWithChangedRate=\
                  np.any(np.isin(reactionindices[:,[0,3]],np.arange(n+1)[reactions[mu]!=0]),axis=-1)
          for index in np.arange(len(reactionindices))[reactionindicesOfreactionsWithChangedRate]:
              (i,j1,j2,k)=reactionindices[index]
              rates[index]=NVec[i]*NVec[k]*pre[i,j1,j2]

      a0=sum(rates)
      mu=0 #reaction index
      aTemp=0
      r2=np.random.uniform()
      while aTemp<=a0*r2:
          aTemp+=rates[mu]
          mu+=1
      mu-=1
      t+=np.random.exponential(scale=1./a0)
      if t>=tmax:
          break
      NVec+=reactions[mu]
  return NVec
