(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.3' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[    119429,       2635]
NotebookOptionsPosition[    116506,       2572]
NotebookOutlinePosition[    116859,       2588]
CellTagsIndexPosition[    116816,       2585]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "This ",
 StyleBox["Notebook ",
  FontSlant->"Italic"],
 " can be used to restore the results of figure\[LineSeparator]",
 StyleBox["\[OpenCurlyDoubleQuote]",
  FontSize->16],
 StyleBox["Dependence of the optimal plasmid copy number on the antibiotic \
concentration.",
  FontSize->16,
  FontWeight->"Bold",
  FontSlant->"Italic"],
 StyleBox["\[CloseCurlyDoubleQuote] (Panel C, E, F)",
  FontSize->16,
  FontSlant->"Italic"]
}], "Section",
 CellChangeTimes->{{3.785732445238887*^9, 3.785732452435005*^9}, {
  3.785732524701559*^9, 3.785732622715519*^9}, {3.7857327208957*^9, 
  3.7857327382568607`*^9}, {3.785741234150372*^9, 
  3.785741236374136*^9}},ExpressionUUID->"a42136b8-3aa0-47fb-a4f4-\
51df235afb8c"],

Cell[CellGroupData[{

Cell["Clear Mathematica environment", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.7857326433428383`*^9, 
  3.7857326481288013`*^9}},ExpressionUUID->"570af557-ecad-40f8-91fd-\
5e4fb83ea7fd"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "\"\<Global`*\>\"", "]"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"Skipped", " ", "Antibiotic", " ", 
    RowBox[{"Concentrations", "."}]}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"skippedAB", "=", "1"}], ";"}], 
  RowBox[{"(*", 
   RowBox[{"Use", " ", "4", " ", "for", " ", "testing", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"~", "10"}], "s"}], ")"}], " ", "and", " ", "use", " ", "1", 
    " ", "to", " ", "obtain", " ", "results", " ", "from", " ", "paper", " ", 
    
    RowBox[{"(", 
     RowBox[{
      RowBox[{"~", "5"}], "min"}], ")"}]}], "*)"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.785740779774435*^9, 3.7857409438891068`*^9}, {
   3.785741083484568*^9, 3.785741098507001*^9}, 3.785741336717936*^9, {
   3.785741492208687*^9, 3.78574149256949*^9}, {3.7857422091197023`*^9, 
   3.7857422096290617`*^9}, {3.78574286294027*^9, 3.7857428927522078`*^9}, 
   3.7857429404793873`*^9, {3.785742975553887*^9, 3.785742977916398*^9}, {
   3.785743086208652*^9, 3.785743129357295*^9}, {3.785743551147554*^9, 
   3.78574355297169*^9}, {3.785744368850461*^9, 3.7857443690351677`*^9}, 
   3.7857448488155622`*^9, {3.7857450767466908`*^9, 3.785745077007942*^9}, 
   3.78574537572283*^9, {3.785745607659137*^9, 3.7857456089324427`*^9}},
 CellLabel->"In[25]:=",ExpressionUUID->"90316cf1-c581-4446-b4e6-b7d11d92456e"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Set parameters", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.785733062339754*^9, 
  3.785733069641982*^9}},ExpressionUUID->"4de71919-c89c-4900-ae99-\
6017d699bbf4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"Set", " ", "N0", "*", "u"}], "*)"}], "\n", 
  RowBox[{
   RowBox[{
    RowBox[{"N0u", "=", ".2"}], ";"}], "\n", 
   RowBox[{"(*", 
    RowBox[{"Set", " ", "death", " ", "rate", " ", 
     RowBox[{"(", "constant", ")"}]}], "*)"}], "\n", 
   RowBox[{
    RowBox[{"\[Mu]", "=", "1."}], ";"}], "\n", 
   RowBox[{"(*", 
    RowBox[{
    "Set", " ", "parameters", " ", "for", " ", "fitness", " ", "curve"}], 
    "*)"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Smax", ",", "Smin", ",", "cMIC", ",", "\[Kappa]"}], "}"}], "=", 
     RowBox[{"{", 
      RowBox[{".5", ",", 
       RowBox[{"-", ".2"}], ",", "1.", ",", "8."}], "}"}]}], ";"}], "\n", 
   RowBox[{"(*", 
    RowBox[{
    "Set", " ", "parameters", " ", "for", " ", "function", " ", "of", " ", 
     "available", " ", "enzymes"}], "*)"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Eta]", ",", "\[Omega]"}], "}"}], "=", 
     RowBox[{"{", 
      RowBox[{"1.3", ",", "0.2"}], "}"}]}], ";"}], "\n", 
   RowBox[{"(*", 
    RowBox[{
    "Set", " ", "parameter", " ", "for", " ", "diffusion", " ", 
     "coefficient"}], "*)"}], "\n", 
   RowBox[{
    RowBox[{"\[Gamma]", "=", "1."}], ";"}], "\n", 
   RowBox[{"(*", 
    RowBox[{
    "Set", " ", "parameter", " ", "for", " ", "plasmid", " ", "cost"}], 
    "*)"}], "\n", 
   RowBox[{
    RowBox[{"c", "=", "0.015"}], ";"}], "\n", 
   RowBox[{"(*", 
    RowBox[{
    "Set", " ", "range", " ", "of", " ", "plasmid", " ", "copy", " ", 
     "numbers", " ", 
     StyleBox["n",
      FontSlant->"Italic"]}], "*)"}], "\n", 
   RowBox[{
    RowBox[{"nRange", "=", 
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "5", ",", "10"}], "}"}]}], ";"}], "\n", 
   RowBox[{"(*", 
    RowBox[{"Set", " ", "range", " ", "of", " ", "antibiotic", " ", 
     StyleBox[
      RowBox[{
       StyleBox["c",
        FontSlant->"Italic"], "oncentrations"}]], " ", 
     StyleBox[
      RowBox[{
       StyleBox["out",
        FontSlant->"Italic"], "side"}]], " ", "the", " ", "cell"}], "*)"}], 
   "\n", 
   RowBox[{
    RowBox[{"coutRange", "=", 
     RowBox[{
      RowBox[{"Range", "[", 
       RowBox[{"cMIC", ",", "2.4", ",", ".02"}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ";;", 
        RowBox[{"-", "1"}], ";;", "skippedAB"}], "]"}], "]"}]}], ";"}], 
   " "}]}]], "Code",
 CellChangeTimes->{{3.785733071717463*^9, 3.7857330893681583`*^9}, 
   3.785733155384768*^9, {3.7857331907989273`*^9, 3.7857332361602783`*^9}, {
   3.785733311739292*^9, 3.785733312650345*^9}, {3.785733481331798*^9, 
   3.78573356047077*^9}, {3.785734288146677*^9, 3.785734326082281*^9}, {
   3.785735247515071*^9, 3.7857352547735043`*^9}, {3.785735299014595*^9, 
   3.785735302838108*^9}, {3.785735454244429*^9, 3.785735455142527*^9}, {
   3.785737797358862*^9, 3.7857378293444557`*^9}, {3.785739268589718*^9, 
   3.785739281537817*^9}, {3.7857393119279213`*^9, 3.78573934003928*^9}, {
   3.78573949774758*^9, 3.785739498582447*^9}, {3.78573991220887*^9, 
   3.785739918620721*^9}, {3.785739984236618*^9, 3.785739984372435*^9}, {
   3.7857407520973263`*^9, 3.785740776899338*^9}, {3.7857408983141203`*^9, 
   3.785740900601679*^9}, {3.7857435214749117`*^9, 3.7857435242595043`*^9}, {
   3.785744627409177*^9, 3.785744627481371*^9}, {3.785744748400462*^9, 
   3.785744748523692*^9}, {3.785744814502714*^9, 3.785744814574629*^9}, {
   3.785744856065254*^9, 3.785744856338455*^9}, {3.785744902463971*^9, 
   3.785744905263625*^9}, {3.785744994256119*^9, 3.7857449977177362`*^9}, {
   3.78574507443543*^9, 3.785745074545298*^9}, {3.785745153961454*^9, 
   3.7857451549853687`*^9}, {3.785745195813661*^9, 3.785745203349577*^9}, {
   3.785745234890122*^9, 3.7857452349629374`*^9}, {3.785745287116251*^9, 
   3.785745287389455*^9}, {3.785745332317809*^9, 3.7857453326917057`*^9}, 
   3.785862889585997*^9},
 CellLabel->"In[27]:=",ExpressionUUID->"32ae2f82-96cf-4734-84db-a3546d4b5524"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Define parameters and functions for \"degradation types\", etc.\
\>", "Subsection",
 InitializationCell->
  True,ExpressionUUID->"656f169a-d280-4184-90ec-b30a49761ee6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
   "Define", " ", "function", " ", "for", " ", "the", " ", "fitness", " ", 
    "S"}], "*)"}], "\n", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"S", "[", "cin_", "]"}], ":=", 
     RowBox[{"Smax", "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"Smax", "-", "Smin"}], ")"}], 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"cin", "/", "cMIC"}], ")"}], "^", "\[Kappa]"}]}], ")"}], 
       "/", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"cin", "/", "cMIC"}], ")"}], "^", "\[Kappa]"}], "-", 
         RowBox[{"Smin", "/", "Smax"}]}], ")"}]}]}]}], ";"}], "\n", "\n", 
   RowBox[{"(*", 
    RowBox[{
    "Define", " ", "function", " ", "for", " ", "the", " ", "amount", " ", 
     "of", " ", "available", " ", "enzymes"}], "*)"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"i_", ",", "n_", ",", "type_"}], "]"}], ":=", 
     TagBox[GridBox[{
        {"\[Piecewise]", GridBox[{
           {
            RowBox[{"\[Eta]", "*", 
             RowBox[{"i", "/", "n"}]}], 
            RowBox[{"StringContainsQ", "[", 
             RowBox[{"type", ",", "\"\<Relative\>\""}], "]"}]},
           {
            RowBox[{"\[Omega]", "*", "i"}], 
            RowBox[{"StringContainsQ", "[", 
             RowBox[{"type", ",", "\"\<Absolute\>\""}], "]"}]},
           {"Null", "True"}
          },
          AllowedDimensions->{2, Automatic},
          Editable->True,
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          
          GridBoxItemSize->{
           "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{1.}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.84]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}},
          Selectable->True]}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxItemSize->{
        "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
         "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.35]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.4]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}],
      "Piecewise",
      DeleteWithContents->True,
      Editable->False,
      SelectWithContents->True,
      Selectable->False]}], ";"}], "\n", 
   RowBox[{"(*", 
    RowBox[{
    "Define", " ", "function", " ", "for", " ", "the", " ", "internal", " ", 
     "concentration"}], "*)"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"cin", "[", 
      RowBox[{"cout_", ",", "i_", ",", "n_", ",", "type_"}], "]"}], ":=", 
     RowBox[{"cout", "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"f", "[", 
          RowBox[{"i", ",", "n", ",", "type"}], "]"}], "/", "\[Gamma]"}], "+",
         "1."}], ")"}]}]}], ";"}]}]}]], "Code",
 CellChangeTimes->{{3.78573357132229*^9, 3.785733574984226*^9}, {
   3.785733703915118*^9, 3.78573373312667*^9}, {3.785733797705576*^9, 
   3.78573379947943*^9}, {3.785733851219039*^9, 3.7857339201477423`*^9}, {
   3.78573397681108*^9, 3.785734010435375*^9}, {3.78573414995365*^9, 
   3.785734154527917*^9}, {3.7857341858423233`*^9, 3.785734259696765*^9}, {
   3.7857343410233393`*^9, 3.785734346595854*^9}, {3.78573438313787*^9, 
   3.785734391714302*^9}, {3.785734434890028*^9, 3.785734627629122*^9}, {
   3.785735231388706*^9, 3.785735234373163*^9}, 3.785735397681242*^9, {
   3.785735433060457*^9, 3.7857354388449574`*^9}, {3.7857354761822*^9, 
   3.785735504344803*^9}, 3.785735537623642*^9, {3.785735573873234*^9, 
   3.785735657086725*^9}, {3.785735693875655*^9, 3.785735710000545*^9}, {
   3.785735854708696*^9, 3.785735856005289*^9}, {3.785736161440322*^9, 
   3.785736173178042*^9}, {3.785737682289612*^9, 3.7857377180014877`*^9}, {
   3.785740369436574*^9, 3.785740369572507*^9}, {3.785744277635207*^9, 
   3.785744277783043*^9}},
 CellLabel->"In[35]:=",ExpressionUUID->"e4d41e4c-d6b8-450f-8506-eafbbcfadc91"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
   "Helper", " ", "function", " ", "returning", " ", "the", " ", "birth", " ",
     "rate", " ", "for", " ", "a", " ", "given", " ", "fitness"}], "*)"}], 
  "\n", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Lambda]1", "[", "S_", "]"}], "=", 
     RowBox[{"\[Mu]", "+", "S"}]}], ";"}], "\n", "\n", 
   RowBox[{"(*", 
    RowBox[{
    "Helper", " ", "function", " ", "for", " ", "death", " ", "rate", " ", 
     "if", " ", "necessary", " ", "including", " ", "costs"}], "*)"}], "\n", 
   RowBox[{
    RowBox[{"\[Mu]1", "[", 
     RowBox[{"n_", ",", "type_"}], "]"}], ":=", 
    TagBox[GridBox[{
       {"\[Piecewise]", GridBox[{
          {
           RowBox[{"\[Mu]", "+", 
            RowBox[{"c", "*", "n"}]}], 
           RowBox[{"StringContainsQ", "[", 
            RowBox[{"type", ",", "\"\<Cost\>\""}], "]"}]},
          {"\[Mu]", "True"}
         },
         AllowedDimensions->{2, Automatic},
         Editable->True,
         GridBoxAlignment->{
          "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{1.}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.84]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}},
         Selectable->True]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{
       "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.35]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}],
     "Piecewise",
     DeleteWithContents->True,
     Editable->False,
     SelectWithContents->True,
     Selectable->False]}], "\n", "\n", 
   RowBox[{"(*", 
    RowBox[{
    "Helper", " ", "function", " ", "returning", " ", "the", " ", "birth", 
     " ", "probability", " ", "for", " ", "a", " ", "given", " ", "birth", 
     " ", "rate"}], "*)"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"p1", "[", 
      RowBox[{"\[Lambda]_", ",", "\[Mu]_"}], "]"}], ":=", 
     RowBox[{"\[Lambda]", "/", 
      RowBox[{"(", 
       RowBox[{"\[Lambda]", "+", "\[Mu]"}], ")"}]}]}], ";"}]}]}]], "Code",
 CellChangeTimes->{{3.78573578016649*^9, 3.7857358267414503`*^9}, {
   3.785736174842517*^9, 3.785736175502653*^9}, {3.785736448101534*^9, 
   3.785736496074402*^9}, 3.785736539038417*^9, {3.7857375500225973`*^9, 
   3.7857375745965843`*^9}, {3.785737630563427*^9, 3.785737630785625*^9}, {
   3.785740313234655*^9, 3.785740313982359*^9}, {3.785743838483667*^9, 
   3.78574392863769*^9}, {3.7857439637239637`*^9, 3.785744001186845*^9}, {
   3.785744291296301*^9, 3.785744293609046*^9}},
 CellLabel->"In[38]:=",ExpressionUUID->"88bddaee-4787-42ec-8c44-eb4e8b11dc00"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"Functions", " ", "for", " ", "birth", " ", "rate"}], ",", " ", 
    RowBox[{
    "death", " ", "rate", " ", "and", " ", "birth", " ", "probability", " ", 
     "respectively"}]}], "*)"}], "\n", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Lambda]", "[", 
     RowBox[{"n_", ",", "type_", ",", "cout_"}], "]"}], ":=", 
    RowBox[{"Function", "[", 
     RowBox[{"i", ",", 
      RowBox[{"\[Lambda]1", "[", 
       RowBox[{"S", "[", 
        RowBox[{"cin", "[", 
         RowBox[{"cout", ",", "i", ",", "n", ",", "type"}], "]"}], "]"}], 
       "]"}]}], "]"}]}], "\n", 
   RowBox[{
    RowBox[{"p", "[", 
     RowBox[{"n_", ",", "type_", ",", "cout_"}], "]"}], ":=", 
    RowBox[{"Function", "[", 
     RowBox[{"i", ",", 
      RowBox[{"p1", "[", 
       RowBox[{
        RowBox[{"\[Lambda]1", "[", 
         RowBox[{"S", "[", 
          RowBox[{"cin", "[", 
           RowBox[{"cout", ",", "i", ",", "n", ",", "type"}], "]"}], "]"}], 
         "]"}], ",", 
        RowBox[{"\[Mu]1", "[", 
         RowBox[{"n", ",", "type"}], "]"}]}], "]"}]}], "]"}]}]}]}]], "Code",
 CellChangeTimes->{{3.785735839687084*^9, 3.785735877479877*^9}, {
   3.7857359319855003`*^9, 3.7857359973620443`*^9}, {3.785736513702689*^9, 
   3.7857365600395393`*^9}, 3.785736897442284*^9, {3.7857373200407267`*^9, 
   3.785737329738834*^9}, {3.785737733504861*^9, 3.78573779202264*^9}, {
   3.78573787859686*^9, 3.785737896933125*^9}, {3.785737972786315*^9, 
   3.785737976587785*^9}, {3.785744132492785*^9, 3.785744231894926*^9}, {
   3.785744478831346*^9, 3.785744500539119*^9}},
 CellLabel->"In[41]:=",ExpressionUUID->"975d7ee5-23ba-4b3c-b689-6c4d2d66b5d9"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Define functions for calculation of establishment and rescue probability\
\>", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.785733000715639*^9, 
  3.7857330095769157`*^9}},ExpressionUUID->"15f85401-bfcf-46cb-b0fc-\
7a4c4795f265"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
    "Define", " ", "a", " ", "module", " ", "to", " ", "calculate", " ", 
     "the", " ", "establishment", " ", "probability", " ", "from", " ", "the",
      " ", "fixed", " ", "point", " ", "of", " ", "the", " ", "probability", 
     " ", "generating", " ", "function", "\n", "given", " ", "the", " ", 
     "plasmid", " ", "copy", " ", "number", " ", "n", " ", "and", " ", "the", 
     " ", "function", " ", "returning", " ", "the", " ", "probabilities", " ",
      "of", " ", "birth", " ", 
     RowBox[{"p", "[", "n", "]"}]}], ":"}], "\n", "*)"}], "\n", "\n", 
  RowBox[{
   RowBox[{
    RowBox[{"Pest", "[", 
     RowBox[{"n_", ",", "p_Function"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Pest", ",", "fi", ",", "f", ",", "q"}], "}"}], ",", "\n", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"p", "[", "n", "]"}], "<=", "0.5"}], ",", " ", 
         RowBox[{"(*", 
          RowBox[{
          "Exclude", " ", "the", " ", "trivial", " ", "case", " ", "where", 
           " ", "establishment", " ", "is", " ", "impossible"}], "*)"}], "\n", 
         RowBox[{"Pest", "=", "0"}], ",", "\n", 
         RowBox[{"(*", 
          RowBox[{
          "Define", " ", "vector", " ", "components", " ", "of", " ", 
           "probability", " ", "generating", " ", "function", " ", 
           RowBox[{"(", "PGF", ")"}]}], "*)"}], "\n", 
         RowBox[{
          RowBox[{
           RowBox[{"fi", "[", 
            RowBox[{"i_Integer", ",", "x_List"}], "]"}], "=", 
           RowBox[{"1", "-", 
            RowBox[{
             RowBox[{"p", "[", "i", "]"}], 
             RowBox[{"(", 
              RowBox[{"1", "-", 
               RowBox[{"Sum", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"PDF", "[", 
                   RowBox[{
                    RowBox[{"HypergeometricDistribution", "[", 
                    RowBox[{"n", ",", 
                    RowBox[{"2", "i"}], ",", 
                    RowBox[{"2", "n"}]}], "]"}], ",", "l"}], "]"}], "*", 
                  RowBox[{"Indexed", "[", 
                   RowBox[{"x", ",", 
                    RowBox[{"l", "+", "1"}]}], "]"}], "*", 
                  RowBox[{"Indexed", "[", 
                   RowBox[{"x", ",", 
                    RowBox[{
                    RowBox[{"2", "i"}], "-", "l", "+", "1"}]}], "]"}]}], ",", 
                 
                 RowBox[{"{", 
                  RowBox[{"l", ",", 
                   RowBox[{"Max", "[", 
                    RowBox[{"0", ",", 
                    RowBox[{
                    RowBox[{"2", "i"}], "-", "n"}]}], "]"}], ",", 
                   RowBox[{"Min", "[", 
                    RowBox[{"n", ",", 
                    RowBox[{"2", "i"}]}], "]"}]}], "}"}]}], "]"}]}], 
              ")"}]}]}]}], ";", "\n", 
          RowBox[{"(*", 
           RowBox[{
           "Define", " ", "multitype", " ", "probability", " ", "generating", 
            " ", "function"}], "*)"}], "\n", 
          RowBox[{
           RowBox[{"f", "[", "x_List", "]"}], "=", 
           RowBox[{"Table", "[", 
            RowBox[{
             RowBox[{"Apply", "[", 
              RowBox[{"fi", ",", 
               RowBox[{"{", 
                RowBox[{"i", ",", "x"}], "}"}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"i", ",", "0", ",", "n"}], "}"}]}], "]"}]}], ";", "\n", 
          
          RowBox[{"(*", 
           RowBox[{
           "Calculation", " ", "of", " ", "the", " ", "fixed", " ", "point", 
            " ", "by", " ", "numerically", " ", "iterating", " ", "the", " ", 
            "PGF"}], "*)"}], "\n", 
          RowBox[{"q", "=", 
           RowBox[{"FixedPoint", "[", 
            RowBox[{"f", ",", 
             RowBox[{"ConstantArray", "[", 
              RowBox[{"0", ",", 
               RowBox[{"n", "+", "1"}]}], "]"}]}], "]"}]}], ";", "\n", 
          RowBox[{"(*", 
           RowBox[{
            RowBox[{
            "Calculate", " ", "the", " ", "establishment", " ", "probability",
              " ", "from", " ", "the", " ", "component", " ", 
             RowBox[{"Subscript", "[", 
              RowBox[{"Q", ",", " ", "1"}], "]"}]}], "=", 
            RowBox[{
             RowBox[{
              RowBox[{"q", "[", 
               RowBox[{"[", "2", "]"}], "]"}], " ", "\n", "since", " ", "q"}],
              "=", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Subscript", "[", 
                RowBox[{"Q", ",", " ", "0"}], "]"}], ",", 
               RowBox[{"Subscript", "[", 
                RowBox[{"Q", ",", " ", "1"}], "]"}], ",", "..."}], ")"}]}]}], 
           "*)"}], "\n", 
          RowBox[{"Pest", "=", 
           RowBox[{"1", "-", 
            RowBox[{"q", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}]}], ";"}]}], "\n", "]"}], ";", 
       "\n", 
       RowBox[{"(*", 
        RowBox[{
        "Return", " ", "the", " ", "establishment", " ", "probability"}], 
        "*)"}], "\n", "Pest"}]}], "\n", "]"}]}], "\n", 
   RowBox[{"(*", 
    RowBox[{
     RowBox[{
      RowBox[{
      "Define", " ", "a", " ", "function", " ", "for", " ", "the", " ", 
       "de"}], "-", 
      RowBox[{
      "novo", " ", "rescue", " ", "probability", " ", "given", " ", "the", 
       " ", "PCN", " ", "n"}]}], ",", " ", 
     RowBox[{
     "establishment", " ", "\[IndentingNewLine]", "probability", " ", 
      "Pest"}], ",", " ", 
     RowBox[{
      RowBox[{
      "exclude", " ", "the", " ", "trivial", " ", "case", " ", "\[Lambda]0"}],
       "\[GreaterEqual]", 
      RowBox[{
       RowBox[{"\[Mu]", " ", "where", " ", "wild"}], "-", 
       RowBox[{
       "type", " ", "population", " ", "does", " ", "not", 
        "\[IndentingNewLine]", "decline"}]}]}]}], "*)"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"Pres1", "[", 
      RowBox[{"n_", ",", "Pest_", ",", "\[Lambda]0_", ",", "\[Mu]0_"}], "]"}],
      ":=", 
     RowBox[{"1", "-", 
      RowBox[{"Exp", "[", 
       RowBox[{
        RowBox[{"-", "N0u"}], "*", 
        RowBox[{"\[Lambda]0", "/", 
         RowBox[{"(", 
          RowBox[{"\[Mu]0", "-", "\[Lambda]0"}], ")"}]}], "*", "n", "*", 
        "Pest"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Pres", "[", 
     RowBox[{"n_", ",", "type_", ",", "cout_"}], "]"}], ":=", 
    TagBox[GridBox[{
       {"\[Piecewise]", GridBox[{
          {"1.", 
           RowBox[{
            RowBox[{
             RowBox[{"\[Lambda]", "[", 
              RowBox[{"n", ",", "type", ",", "cout"}], "]"}], "[", "0", "]"}],
             "\[GreaterEqual]", 
            RowBox[{"\[Mu]1", "[", 
             RowBox[{"n", ",", "type"}], "]"}]}]},
          {
           RowBox[{"Pres1", "[", 
            RowBox[{"n", ",", 
             RowBox[{"Pest", "[", 
              RowBox[{"n", ",", 
               RowBox[{"p", "[", 
                RowBox[{"n", ",", "type", ",", "cout"}], "]"}]}], "]"}], ",", 
             
             RowBox[{
              RowBox[{"\[Lambda]", "[", 
               RowBox[{"n", ",", "type", ",", "cout"}], "]"}], "[", "0", 
              "]"}], ",", 
             RowBox[{"\[Mu]1", "[", 
              RowBox[{"n", ",", "type"}], "]"}]}], "]"}], "True"}
         },
         AllowedDimensions->{2, Automatic},
         Editable->True,
         GridBoxAlignment->{
          "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{1.}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.84]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}},
         Selectable->True]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{
       "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.35]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}],
     "Piecewise",
     DeleteWithContents->True,
     Editable->False,
     SelectWithContents->True,
     Selectable->False]}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.785732923559453*^9, 3.785733031496773*^9, {3.785733659250031*^9, 
   3.785733678825623*^9}, {3.785737025994038*^9, 3.7857370523574867`*^9}, {
   3.785737117212419*^9, 3.785737302116993*^9}, {3.785738685483532*^9, 
   3.785738754896576*^9}, {3.785738845926865*^9, 3.7857388974633007`*^9}, {
   3.785744037591073*^9, 3.785744096579754*^9}, {3.785744240697887*^9, 
   3.785744262197443*^9}},
 FontSize->12,
 CellLabel->"In[43]:=",ExpressionUUID->"fd1f9ca5-eb57-40a0-9bd2-aa6127eccde8"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Calculate rescue probabilities \nfor the three types \nPanel C \
\[Dash] ", "Subsection"],
 StyleBox["Relative ", "Subsection",
  FontSlant->"Italic"],
 StyleBox["(Relative number of mutated plasmids determine the degredation \
rate)", "Subsection"],
 StyleBox[", \n", "Subsection",
  FontSlant->"Italic"],
 StyleBox["Panel E ", "Subsection"],
 StyleBox["\[Dash]  Absolute ", "Subsection",
  FontSlant->"Italic"],
 StyleBox["(Absolute number of mutated plasmids determine the degredation \
rate)", "Subsection"],
 StyleBox[", \n", "Subsection",
  FontSlant->"Italic"],
 StyleBox["Panel F ", "Subsection"],
 StyleBox["\[Dash]  AbsoluteCost ", "Subsection",
  FontSlant->"Italic"],
 StyleBox["(like Absolute and fitness is deteriorating with increasing PCN ", 
  "Subsection"],
 StyleBox["n", "Subsection",
  FontSlant->"Italic"],
 StyleBox[")\nfor range of antibiotic concentrations ", "Subsection"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["c", "out"], TraditionalForm]],ExpressionUUID->
  "3eec8c61-9b6e-46b0-8d09-047f124a7fbb"],
 " (columns)\nand for various plasmid copy numbers ",
 StyleBox["n ",
  FontSlant->"Italic"],
 "(rows)"
}], "Subsection",
 CellChangeTimes->{{3.785740530522457*^9, 3.7857407406210957`*^9}, {
  3.785740965503007*^9, 3.785740978803155*^9}, {3.785741961834478*^9, 
  3.7857419681973667`*^9}, {3.7857460822876368`*^9, 
  3.78574608332412*^9}},ExpressionUUID->"3da18ec3-fc87-4e5f-9d77-\
52b6def5cf92"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
   "Helper", " ", "function", " ", "to", " ", "calculate", " ", "data", " ", 
    "for", " ", "ranges"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"CalculateRescueProbabilities", "[", "type_", "]"}], ":=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"cout", ",", 
        RowBox[{"Pres", "[", 
         RowBox[{"n", ",", "type", ",", "cout"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"n", ",", "nRange"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"cout", ",", "coutRange"}], "}"}]}], "]"}]}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
    "Helper", " ", "function", " ", "to", " ", "show", " ", "result", " ", 
     "table", " ", "as", " ", "a", " ", "grid"}], "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"GridData", "[", "t_", "]"}], ":=", 
    RowBox[{"Grid", "[", 
     RowBox[{"MapThread", "[", 
      RowBox[{"Prepend", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Prepend", "[", 
          RowBox[{
           RowBox[{"t", "[", 
            RowBox[{"[", 
             RowBox[{"All", ",", 
              RowBox[{"1", ";;", 
               RowBox[{"-", "1"}], ";;", 
               RowBox[{"Ceiling", "[", 
                RowBox[{
                 RowBox[{"Length", "[", 
                  RowBox[{"t", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], "]"}], "/", "10"}], 
                "]"}]}], ",", "2"}], "]"}], "]"}], "\[IndentingNewLine]", ",", 
           RowBox[{"coutRange", "[", 
            RowBox[{"[", 
             RowBox[{"1", ";;", 
              RowBox[{"-", "1"}], ";;", 
              RowBox[{"Ceiling", "[", 
               RowBox[{
                RowBox[{"Length", "[", 
                 RowBox[{"t", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], "]"}], "/", "10"}], "]"}]}],
              "]"}], "]"}]}], "]"}], ",", 
         RowBox[{"Prepend", "[", 
          RowBox[{"nRange", ",", "\"\<\>\""}], "]"}]}], "}"}]}], "]"}], 
     "]"}]}], "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{"Helper", " ", "function", " ", "fr", " ", "plots"}], "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ListPlotData", "[", 
     RowBox[{"t_", ",", "type_"}], "]"}], ":=", 
    RowBox[{"ListPlot", "[", 
     RowBox[{"t", ",", 
      RowBox[{"PlotLabel", "\[Rule]", "type"}], ",", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
        "\"\<Antibiotic concentration \!\(\*SubscriptBox[\(c\), \
\(out\)]\)\>\"", ",", "\"\<Rescue probability\>\""}], "\n", "}"}]}], ",", 
      RowBox[{"PlotTheme", "\[Rule]", "\"\<Scientific\>\""}], ",", 
      RowBox[{"PlotRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}]}], ",", 
      RowBox[{"PlotLegends", "\[Rule]", 
       RowBox[{"PointLegend", "[", 
        RowBox[{"nRange", ",", 
         RowBox[{
         "LegendLabel", "\[Rule]", 
          "\"\<Plasmid copy \\nnumber \
\!\(\*StyleBox[\"n\",FontSlant->\"Italic\"]\)\>\""}]}], "]"}]}]}], 
     "]"}]}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.785741539820969*^9, 3.785741578034748*^9}, {
  3.785741624010651*^9, 3.7857416450240593`*^9}, {3.785741686990403*^9, 
  3.785741724176711*^9}, {3.785741755682913*^9, 3.785741817829784*^9}, {
  3.7857418511588182`*^9, 3.7857419524467278`*^9}, {3.7857420458682013`*^9, 
  3.7857420548628607`*^9}, {3.785742089416502*^9, 3.785742127840961*^9}, {
  3.785742809056419*^9, 3.78574285637779*^9}, {3.785742950055191*^9, 
  3.785742965654996*^9}, {3.785743021121395*^9, 3.785743072870613*^9}, {
  3.7857432912793303`*^9, 3.785743301740505*^9}},
 FontSize->12,
 CellLabel->"In[46]:=",ExpressionUUID->"49699357-0200-4822-8eb8-c49da288e8a6"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"type", "=", "\"\<Relative - Panel C\>\""}], ";"}], 
  "\[IndentingNewLine]"}], "\n", 
 RowBox[{
  RowBox[{"t", "=", 
   RowBox[{"CalculateRescueProbabilities", "[", "type", "]"}]}], 
  ";"}], "\n", 
 RowBox[{"GridData", "[", "t", "]"}], "\n", 
 RowBox[{"ListPlotData", "[", 
  RowBox[{"t", ",", "type"}], "]"}]}], "Code",
 InitializationCell->False,
 CellChangeTimes->{{3.785737393989255*^9, 3.785737447767462*^9}, {
   3.785738034736822*^9, 3.7857380357523317`*^9}, 3.785738095216279*^9, {
   3.7857382896725693`*^9, 3.785738345607823*^9}, {3.785738914627738*^9, 
   3.785738968577039*^9}, {3.785739002530993*^9, 3.7857390648211946`*^9}, {
   3.785739103123219*^9, 3.785739122035618*^9}, {3.785739187125795*^9, 
   3.785739187222313*^9}, {3.785739226713036*^9, 3.785739227385872*^9}, {
   3.785739357591921*^9, 3.785739396705508*^9}, {3.7857394671327353`*^9, 
   3.785739582573188*^9}, {3.7857396198021383`*^9, 3.785739764102675*^9}, {
   3.7857397983701897`*^9, 3.785739892994609*^9}, {3.785739926185313*^9, 
   3.785739939584598*^9}, {3.7857399704295883`*^9, 3.7857399709852533`*^9}, {
   3.785740991106814*^9, 3.785741046744586*^9}, {3.785741393343977*^9, 
   3.785741477932468*^9}, {3.7857415192123528`*^9, 3.785741535398093*^9}, {
   3.7857419954984407`*^9, 3.785742087229521*^9}, {3.7857421300288477`*^9, 
   3.785742173005332*^9}, 3.785744518204702*^9},
 CellLabel->"In[49]:=",ExpressionUUID->"337e31d1-b7bc-428f-b8f2-6a7c7d0ef4c9"],

Cell[BoxData[
 TagBox[GridBox[{
    {"\<\"\"\>", "1.`", "1.16`", "1.32`", "1.48`", "1.6400000000000001`", 
     "1.8`", "1.96`", "2.12`", "2.2800000000000002`"},
    {"1", "1.`", "0.37306183921189573`", "0.2700019089114909`", 
     "0.23575053038825522`", "0.21043975652374847`", "0.17801433878885708`", 
     "0.1319987191690467`", "0.07289725066763797`", "0.008008856910418083`"},
    {"2", "1.`", "0.4517190223633504`", "0.30925977264725035`", 
     "0.2413178432005374`", "0.19060118173170537`", "0.1468230338234573`", 
     "0.10293840810062915`", "0.055314967320961306`", 
     "0.0060185377334980306`"},
    {"5", "1.`", "0.5247925866065624`", "0.2925584091330953`", 
     "0.18578590659788752`", "0.12372130379198754`", "0.0824521171625826`", 
     "0.05143078383259614`", "0.025486217410283674`", "0.002649039789067942`"},
    {"10", "1.`", "0.525490293718472`", "0.23567175395128326`", 
     "0.12153604645523719`", "0.06742373218299447`", "0.03821582470694029`", 
     "0.02072791856929057`", "0.009174926548206175`", "0.0008767505379972951`"}
   },
   AutoDelete->False,
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output",
 CellChangeTimes->{3.7857454168076*^9, 3.785745783079137*^9, 
  3.785862971086095*^9},
 CellLabel->"Out[51]=",ExpressionUUID->"999f6abb-2002-4bf2-97da-43e0d9dcaf25"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{}, {{{}, {
        Hue[0.67, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.9, 0.36, 0.054], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJxF039MlHUcB/CHC4MoRRTMhSeZf0SOhATXcML7m6hQkSkSiIPFGLI2NY5p
Ew9/nJrODTav6OAgXJaIuBJOTXaJKzy8g4MDTqVOEgVaLBSIg+O4Xxznc899
is/27Nlnz/be9/N5fZ9VeYVpe0QcxyXzj/ftKzP+e6epV54e3WHGarUi47ec
f1ErSQgyXTRjjTt4aegrTzETmX1OYzXj/brjT9bq/kbKkDS0IXkS7Q9d28a1
Q6iu8tYkAjWFW4eTBjC2wxs4iZqwI4s/F/cjkU+TJEzhTP2qzWdu9YEP4xOn
0JR4ozVx0AQhbmgK8rCe1tzP/kCsUBYYP9ZySfG/gz8cH2jBs+OTAa+91wtv
WpDJgn2Gj/SxBx7gLe/xIqchVUhFybr7KPHGSadRcePY7rzY++gyeGsazr0d
rg037yFCCLSi9nDksqbkeyjyjiuxQlUzrq38xwjheBoroj6cyPrgayOEuNAZ
DJ5NeHxzixEFQs2g+Mpl9RucEcL61DMwhpW/miLrgTBukA2p3XGuRS/1QDhe
tg0qRaHzK2U3hLgGG4oj/PVL3u4mBzuYJ/3oIm0XedhRb/ozY92nXeRhR3qT
0pnmNJCHHeaTLRV7FAbycEDmGDoljTaQhwPm5orQX9o6ycOBuODezOycTvJw
YkX4ixdUEx2QCwM78WVq2o8hxzrwl+DhxJHv+7c/DehAnODhgiyzoKahTE8e
LhzMqdwy/LKePFwwmlYeEJ9uJ49ZqLf3NVtsbeQxi427S7NC8tvIYxYx1fUn
ovQ68nBDfi2uL/BNHYRxJW4oPWsch45qycONxeNiR5bhLnnMYcQ9az2/9C55
zMFQnVi6aWcrecxh13fL12eUasjDg5/2Bpfbmu+QhwdjFvvJT4ZbyMOD29F1
CxsDWsiDYzEHv0Dd1l9pHo4pxVVhrOg2+XAsJTyzaoH8Fs3HsZGyd+PHLqrJ
i2Ptl5QyUUMTzcsxu/yHXfsbfyY/jjG7YjT/ynWa34/JSmLi3zmvIk8/xilL
HqUuv0r74L/nRkdsmqgnX77/VhLjr6ml/fgxlXWJOKP8Ann7sVzdioGRTiXt
S8S418sN4Yfl5C9irDLkm6kCGe2P/07luw/zvW+f873vfsz3vv2+8H/v+3/n
++et8A4s
         "]]}, {
        Hue[0.9060679774997897, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.365248, 0.427802, 0.758297], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJxF02lMU2kUBuBbHLUQtyhxZtRIcCFtIIrW8YeGvh9xo+IGiTFRIrigZtBY
o3EpRusPEiMuqKBgBFwQRaXGLTSgWEpBQesCOlAxDkvVGjQWWtpC6cXb26Oc
5ObmJDdvvnOe74Zv3JmYGsRx3BLh8b8DZcevd6J+ckZngh2xOvWKnK0/UKSO
CWm6akdKY8ua6MZOuGRJp4w9dmhzmHZ+iA1xbZpQ3ZIuSNM0O28Ff8KFPH91
gU8+XdzyuB3fEvyBXXBlv7WEelqhFNLUMd1IXL2mSB7xP4QwIbEb44cHfXHZ
PkCMa+vG2MUyt3FcCxRiOdC/J3z0ueMWCIcTAh0YtmNSRT1rhj8tpMkBk/14
WeWUJsj9x5M5cdK74vpK2X9I98dpnJgxpIa3qN7B/MJfTpxTxcszj7xFmBjY
AytL1n6tb8Qu/7jqHszOZrmHpzRCPJ6xB7rzirDCjAaIcaEu8KoPekX3G2wR
y4XywozWS1veQFyf3gXZ3IgT19teQxw3xI0JTwrSEja+hni8JDcWju/QGF69
ghinc0O7TVmv7XhJDh4YKsOcXt5MHh5IFVUd+8LN5OGB+lDShLVLX5CHB38c
GX1m0oHn5NGL5pLLsYtK68mjF9szx1iTP9eRRy+ctojS3ql15NGHou+j8gpS
nyFLHLgPWT59cPPNp2gXPfogzXT8G+WoxRzRw4vou6vyw5S15OHFwbPT7YrM
GvLwIsVxLL3TYiKPfjwYarpfJzeRRz+ksnV/rtZUk0c/DJNVrX89N5KHD1H8
juLiiUaI46p9mBa77/P5tCry8CHuwLjNp8oN5MHjUY58wzypgTx4aB/dj796
o5I8eMStH1k6J+YxeQxgTG1SlNlcQR4DyN0cb4tcW04eA2j+aFJFtuvJg2O2
Pf+MqEoto3k4Jl0e+dFnfUg+HDPEbsofkfKA5uOYvaVi9qyme+QlfO/+O/Tp
0rs0L8fUBZYvh/beIT+OZVmVK5WJt2l+CUvJziv0yEvIU8KiG6ovuvlrtA8J
y+prmJnfcIV8Jay1xFhWsayQ9iP0q0Yu2OvOJW8JM6zb/T79xhnaVxBjEWad
qeYo+QcxLri601q8n/Yn9FSB+zDYB/Y52Afux2Af2O+Q333g/x3sfwK3SxLv

         "]]}, {
        Hue[0.1421359549995791, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.945109, 0.593901, 0.], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJxFkw1MjHEcx5+7M+Mwb0cKoY6dZgm3SZbvg7liNNnMS3kr571cLCo2LWUz
jfMaR4lehCJvvUxKV0rU9KoyUrcwQnedq+t689xzP/ptz/777r999/t+P/9n
RsDBdUohwzBe3Gc9baPHv3NdjmNMm68eCaOku+3a25Gs8hTXJ+lRFJBfJIr5
jU6Z/zmtSY9TWs2I4to2eLdESB54GaBqyXBsFXyH5pp1DHAds1R+98hX/PS1
GhowZ2S0RLaiFUs4N5VnB+q7jk9NVerAmXGOHdCNvVxt/7YZvF1LB+59vl2x
sLoJC/gx4oNI3nh530dwy3GGRgwtcnBK9fkAq5u43ojgZxOz3EMaMNu6nuwP
LhwJbGsqfY9jVruIP9CotP4+q+pQUW4d7j7MNbzVUINpvKEJ0m2N3xS51Qix
xlWZUL5cWuZwowr8eloTmgojNlZcqQRvJ+mELKLNuWb1O+zipxPR19e4tKsr
wNeX0wnVHMe8EtNb8HHFXThQPm9W9dE34Nfz70Lsqx0eIyRl4O0edIFNPiSt
Ky0lDmYMi7ux97u6hHiYUeyX8Ms76BXxMEPea7Rz8ysmHmYw2auVozYVEY9u
uG/VNYQrtcSjG5lZi1vmnygkHt14uiZqy4SUl8TDgswvmuEL0gug5gNbEBaS
wdx0yoeO52GBd3iwS+79PMh5Hj3YXpASY1jxnHj0IC18b1Rjey7x6IF+EpMw
PS2HePTCfdmZk+eDsolHL9w+NV8fjSziwd0Ln1guTX5GPPpw1XOuxk34FHxc
VR8i6/IahnQ8Jh59SJTEuip+PCIe/fCOV4zz+JFJPPqhtqsVH9A9JB79qDTv
P+nZl0E8BsBE+YneOacTjwGoZzq/cFp/j3gMgN1gPL5enUY8GHZ6WLcmvjaV
8jDs9kWzLStnpBAfhtXnp4kLQ5MoH6cDBa9PVN0iXgxbmXAwfZI8kfIyLHtY
LlN6xBM/Tp92sB9tr6H8ArZ5s3lKqSiOeApYRvZtmcJykfrg9IaddYG9auLL
6fH2d6SSWOqH04uNmhzFKeItYNnow5u16kjqS8hGGs9+9PoSSvyFLJO5do/q
iZL64zSN7T0Malufg9r2Pga1rV/Rf237fwf1X1hEA30=
         "]]}, {
        Hue[0.37820393249936934`, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.645957, 0.253192, 0.685109], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJxFkwtIU3EYxe8UpdbDVVJDzBe9jAzyURTVuUW6mKZpDywXPUwhwbqWIdhr
RqNSIjXC0prrYdhLLTMfvYZhL5strTSK1Mg0LZzTualTu7v7yg8ufw78OXzn
/P7Xe9e+qDgHhmFk/Gc77WPAvzOqwkPVFWnAn5CdM9yGu3GdWyFuvGbAkk/7
dcGh3eifpzhbbTKgbLY5LnbxH6xtTXUtkvXARRbtG5raidyLtumBOmdxiuJ+
O35H2gx7cKrNLzvGsw0reTduhREv9Jfr3D98B2/GOxqxWbHc6b2+BYJdqxEL
v35Ol3p8Q4AwvchO63iyZdIX8Mvxhr2YGiGvOba9CTY3cWMvWC7PEu78Cb62
9eb1QdK8sUNpasAhm11qH7bmM8vu+NRD99Y2ffAIqVRPUOnhKRiakPyz3XRk
fh2SbHE5E7inMn+f3FoI61Wb8KDzuTw9/DUEO9d+vHK4kt608iXihemHxHGj
iyq2BkJ9Ff0obH26cNbD5xDiis0Im/OMXbKoGsJ6CjN216jbnfRaCHZFZkgP
vlNa3Z8RBws0XqtjguofEw8LlKWSbRcqq4iHBa+Km+v1ugriYUFh3WhykLic
eAyAzT6aXJxQRjwGoP0w7XxBVynxGMC42dpLnafvE49BtKwqPVG0+h4yhcC8
/vLDmDStBN8FHoNgs4ICzb13ESjwGIIkSrJD3XGbePBav3ZfqOgW8RiCMte5
QbeskHhYwUXr3GTZN4iHFRonPbydCoiHFZKbl+TOWdeIxzC80uI8PwdchRCX
G4ZmT2JwcZeGeAyDsT4an5OYTzxG4LWrLMV/7mXiMQJNBKdZL8ojHrzmpOEb
Bi4Qj1GwSQb3pok5xGMU2pmpU0qXniceo/Dqa234deQc8WBYjXNYSnFjFuVh
WO0al/IdwZnEh2HZ021+oowzlI9hmZKEOwl304kXf/+WX0RG90nKy2uxOnhB
hIr4MazS+KR23ZvjlF/Eag8ZH/tsVxJPEavcdPijVHqY+uD19MkKv/wU4svr
oTnRHesPUD8illHJ53dt2ku8ef1IjrTN8dSXA8u0VCZWYhvx5/XvnKTyqjDq
j9c09vcwpu19jmn7+xjT9n4d/2v7/zum/wLgvv9p
         
         "]]}, {}}}, {}, {}, {}, {}}, {
    DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {0, 0}}, AxesOrigin -> {0.978125, 0}, 
     PlotRange -> {{1., 2.4}, {0, 1}}, PlotRangeClipping -> True, 
     ImagePadding -> All, DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0.978125, 0}, DisplayFunction :> 
     Identity, Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"Rescue probability\"", TraditionalForm], None}, {
        FormBox[
        "\"Antibiotic concentration \\!\\(\\*SubscriptBox[\\(c\\), \\(out\\)]\
\\)\"", TraditionalForm], None}}, FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {{0}, {0}}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], LabelStyle -> {FontFamily -> "Times"}, 
     Method -> {"CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, PlotLabel -> 
     FormBox["\"Relative - Panel C\"", TraditionalForm], 
     PlotRange -> {{1., 2.4}, {0, 1}}, PlotRangeClipping -> True, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {0, 0}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"1", "2", "5", "10"}, "PointLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               StyleBox[
               "\"Plasmid copy \\nnumber \
\\!\\(\\*StyleBox[\\\"n\\\",FontSlant->\\\"Italic\\\"]\\)\"", {
                FontFamily -> "Times", FontFamily -> "Arial"}, Background -> 
                Automatic, StripOnInput -> False]}, {
               TagBox[
                GridBox[{{
                   TagBox[
                    GridBox[{{
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.9, 0.36, 0.054], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.9, 0.36, 0.054], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.365248, 0.427802, 0.758297], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.365248, 0.427802, 0.758297], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.945109, 0.593901, 0.], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.945109, 0.593901, 0.], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.645957, 0.253192, 0.685109], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.645957, 0.253192, 0.685109], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                 False, GridBoxItemSize -> {
                  "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                 GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
                "Grid"]}}, GridBoxAlignment -> {"Columns" -> {{Center}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
           Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Times", FontFamily -> "Arial"}, Background -> 
         Automatic, StripOnInput -> False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"PointLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.9, 0.36, 0.054], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.6000000000000001, 0.24, 0.036000000000000004`],
                     FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.9`", ",", "0.36`", ",", "0.054`"}], "]"}], 
                    NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.9, 0.36, 0.054]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.9, 0.36, 0.054], Editable -> False, Selectable -> 
                    False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.365248, 0.427802, 0.758297], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.2434986666666667, 0.28520133333333336`, 
                    0.5055313333333333], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.365248`", ",", "0.427802`", ",", "0.758297`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.365248, 0.427802, 0.758297]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.365248, 0.427802, 0.758297], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.945109, 0.593901, 0.], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0.6300726666666667, 0.395934, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.945109`", ",", "0.593901`", ",", "0.`"}], 
                    "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.945109, 0.593901, 0.]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.945109, 0.593901, 0.], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.645957, 0.253192, 0.685109], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.430638, 0.16879466666666665`, 0.45673933333333333`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.645957`", ",", "0.253192`", ",", "0.685109`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.645957, 0.253192, 0.685109]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.645957, 0.253192, 0.685109], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
           RowBox[{"Joined", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{"False", ",", "False", ",", "False", ",", "False"}], 
               "}"}]}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{"FontFamily", "\[Rule]", "\"Times\""}], "}"}]}], ",", 
           RowBox[{
            "LegendLabel", "\[Rule]", 
             "\"Plasmid copy \\nnumber \
\\!\\(\\*StyleBox[\\\"n\\\",FontSlant->\\\"Italic\\\"]\\)\""}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.7857454168076*^9, 3.785745783079137*^9, 
  3.785862971382504*^9},
 CellLabel->"Out[52]=",ExpressionUUID->"7b87aef2-584f-4c17-a24a-2642f5ae8895"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"type", "=", "\"\<Absolute - Panel E\>\""}], ";"}], "\n"}], "\n", 
 RowBox[{
  RowBox[{"t", "=", 
   RowBox[{"CalculateRescueProbabilities", "[", "type", "]"}]}], 
  ";"}], "\n", 
 RowBox[{"GridData", "[", "t", "]"}], "\n", 
 RowBox[{"ListPlotData", "[", 
  RowBox[{"t", ",", "type"}], "]"}], "\n"}], "Code",
 InitializationCell->False,
 CellChangeTimes->{{3.785739974864916*^9, 3.785739975561471*^9}, {
   3.785741017996529*^9, 3.7857410514187717`*^9}, 3.785742192197216*^9},
 CellLabel->"In[53]:=",ExpressionUUID->"99634223-9886-40e3-99ee-4948c143dfa6"],

Cell[BoxData[
 TagBox[GridBox[{
    {"\<\"\"\>", "1.`", "1.16`", "1.32`", "1.48`", "1.6400000000000001`", 
     "1.8`", "1.96`", "2.12`", "2.2800000000000002`"},
    {"1", "1.`", "0.05402459925136638`", "0.`", "0.`", "0.`", "0.`", "0.`", 
     "0.`", "0.`"},
    {"2", "1.`", "0.23868946589252638`", "0.05473352361722772`", "0.`", "0.`",
      "0.`", "0.`", "0.`", "0.`"},
    {"5", "1.`", "0.455144620868694`", "0.22591843886413854`", 
     "0.128466640697528`", "0.07368912617103096`", "0.03608265750808226`", 
     "0.006501904279081372`", "0.`", "0.`"},
    {"10", "1.`", "0.6719426635700853`", "0.37835252939434794`", 
     "0.233029702167732`", "0.1520417215886165`", "0.10287447360382318`", 
     "0.0711499759768991`", "0.04970865674736569`", "0.034620697307950365`"}
   },
   AutoDelete->False,
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output",
 CellChangeTimes->{3.7857454304224453`*^9, 3.785745841252161*^9, 
  3.785863028360537*^9},
 CellLabel->"Out[55]=",ExpressionUUID->"79668249-8c14-4943-b1dd-c63117e37147"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{}, {{{}, {
        Hue[0.67, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.9, 0.36, 0.054], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJxd0z9IQlEUx/GXEYTQEEotkRQJNudqpylr1KnBIaQa60W0GARBTQ1Fm21m
BRE4RdgQiEtLRUPkEqHR0j94Wtof//XuPXh/dC+IfJbDPferA9GF8KzDMIyg
/RHffCxqfYfT/RuvIYui64dPy6vPtG8GnLmkRdf+s4elvTxVfJGtbNminnHX
TPLljiYKMXcqWCSjd951O3VDu3FxijTd1d3RuXlBbyExsEhDO+HBxEmGRu1p
ZqBEmaBn0nlwSvYwe6LtxPt539ExyXGFEuXXvN77eJxG5Pmg1k3ty9kDYTHN
mYOHxfV8n8orYlwMvroUB/bIgWXlRbGuCcvrZWE5zl1RnpMHls+XhuW6zi9l
eb0ILMelYD7fytwD5h4w94C5x48y94C5B8w9fpW35cLwo+wB+2WPqtajqvWo
aj1qWo+a1qOm9agry3XNutajrvVoaD0aWo+G1qOp9WhqPZpaD2Ps/z4w94F5
P5h7wbwvzP1g3r9NmXvC/B4w94X5fWDuDfN7OZS5P8zvB/PvAeb3hPn3AfP7
tivz/xf+AyQtMRs=
         "]]}, {
        Hue[0.9060679774997897, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.365248, 0.427802, 0.758297], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJxd00tIVGEYxvGTScjYooW7wogWWaugWURpT5swmBYpWFITiGUUCU6FKAaR
VIsQKYICRSFIWpVEUQmCDFohkd0gpiAEJy3pxpmr2WWmc74Hv4fOB8Pht3k5
3/s/s665rb6lxHGcWu/nP3lcLD3rRyovfq1z8aY7Prut/AeGYjWhxE0XZ48s
tB6e+ox8VfTyeM5Fe/Jgy4PHSeye6aoYrk3hwqNXX+b3TaO/zz8pVN86vWFy
1Xt8q/MHpnBgbu/ou/K32OFNi9Wk0fwxcejU9Gt4w7yJacxWZ6NNvS9gxs2k
Ubp6667t4WfYYk4Gd4/3jc11PoX3ct7ADDa/HOi5Fp6APy2UyGDy3JqxlXvi
2Oi/XlUWjQ2tdwb3j+KMP64ri84n+d6OnoeYeu6fLNzv568WKu9hrRmYw43r
HyJlK27jpH/dWA7Op7Jj9xuHYF5vPId46kSkNNIPM64iD2dTW9P67ks4ak4e
S5s06xuRzXVDC9bm9aKyGTcs8/y0Zg+ZPWT2kNlj0Zo9ZPaQ2eOX9RVzYTlp
eshh0+O3NXvI7CGzxx9r9pDZQ2aPv9bmujGZPWT2KFizRyHQoxDoUQz0KAZ6
FAM9nJ3/30dmH5n3k9lL5n1l9pN5/2XW7ClzHzL7ytyPzN4y91Vizf4y9yfz
e5C5T5nfh8z9Lrfm/1f+B1PRXFk=
         "]]}, {
        Hue[0.1421359549995791, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.945109, 0.593901, 0.], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJxFk38s1GEcx4+s7DbFWD+0cjZ/nLW0JkbD28YmNzSm/6zJkGzqrkyilUrL
skatK4yVWCNzU9KdQyGNiRzJYTFEJM2dcxzH6Xvf5+H5bN89e+27vff5fF7P
455wJSbJViAQhHGf9SSlw/YZozp6/2+0DouW6c2B8UVUSgOF2godJhsMl2//
/IcVcVxBu1EHm89RDwsL5nFmMstFEabHk7R8UX7sLEqKraXH1VSbQW3eNBai
rYF61IxPNM2dmEIQlyYNXEJe+5hXrd8EuDAucQkCn6TZ/e5j4OMml5DuOWd3
9+wovPkyIGNRXFH9SwuuOS7QAOERgcPJ7h+wpgm1BnjYXfc5sPwdntb2xMuY
3nLGm3MDyLbGZS2jx8mcppzRoLfHWsuoOjzSMu7fBzc+0Iin+4L8Z+Z7ILOO
KzXiZtNshGmwG3x77UZkuja1hOi6wMe5rED1tXgh3LcTyXytoOPeqlNp+Rfw
61OtQHxeoag43gF+XOEqAvqfqTKH2sG3F7cKv2MfQtqK2sDHKVaRLvKVFMla
qQcT/Pb2KgzNH6kPE6QO6rJkdTP1wbE4Xl2rVVMfJhTu1jUmODVSH2uoCpZH
5SYqqY81aPqEfSnfGqiPNcQ7PQ+tj3hPfawjr34tQTL2DoX8wOvI2XhgH33r
LaZ4H+uwH1aa3LzqcIr3YYYmaNTjzlAt9WHGxKUXZZqUGurDDFG4vPmPYzX1
sYE6fWtOf89r6mMDmo4GmaakkvrYgGPib+e0G6+oj02ILHGfki++BD+udBPx
uu5rFzpLqQ/uvzLbPDpaRH1YECypOT24R059WCAa6crwjnxMfViQ88g5JjQ2
n/rYQmtkqcw+IJf62EKOfLiipTyL+tiC4JCr5GB4KvUhCN5+KWQexsQPYzIf
Y+KLMZmXMfHHmMxvs8PEJ2OyD8bEL2OyH8bEN2OyL9sdJv4Zk/0xJveBMdkn
Y3I/GJP97tph8n4Z/wd6jtfK
         "]]}, {
        Hue[0.37820393249936934`, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.645957, 0.253192, 0.685109], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJw1031ME3cYB/BrWZySTVCPBUmmdRkGdTEECbhF9/0N44ooTgkCGhNRAWFj
GyIjjhltNs1MNkPTTLMx2UBRi2ipvLjioJbKfGESQARcDW91E5WCLfSN0tJd
b8+e5HJ/3N03z/N8frds3+cp2VKO4+TCFbj/V1b8f0/RLTkxtt0KSe0XjdX+
l6gqWB/cf96K6PdMbYrbL+GM2l1qdFghOxG/uLN1AokjJbxGbsORPzuTkvhx
lP0UKBuc8r4YuWwMlu2BQBsOVJZvXhT5HO8LaQXrJ5F/zPRW5AejEMKExEm0
JJQ9PaX4B2LcyCTODh/5eufTJ1gj1hSiuzdoDx80Q2hOCJxCxnLp/neWjiCQ
Ftw/hXVXr4zvejSEFYH2ouwYTs0tzLs8gK8CcSV2mH6Id+RpHqPjfqDsiJ17
ZnZw8C8sFQMdcJvnfDse9wgHA+MWOKC/pbKk1PVBbM/owOHOzSE7knshxvFO
XD/ZhGuvPESOWE4YX78X8X3vA4jr0znRkH88cn9LN8Rxg12ICbqwk7vRBbG9
3S7whjh+SNUJMU7jQvOW0QcvejrIwY22du/zkNj75OGGfW59mFLbTh5u5C5Y
a3V9eI883CjqN6nDHHfIYxqhyZoQm+42eUxD2beluU35B3lMI6t26zJ3SRt5
eGAdseREFN+CUhzYg2evFU0MKIwwix4eNGSYsvmyVsSKHjNI9SWk1xgM5DED
WWu2PGX0JnnMYK2lrzE1S08eXoRHOe/qnc3k4YWy+Mshvvx38vBC1qLbMLPj
Bnn4oD632h4ja4I4boEP4b6e9jOe38jDB9lgviv87+vkMYtnY3uMhseN5DEL
7ak5perhBvKYxXCRKsxrqycPPxhf2+GcX08efuSenvjGHFdHHn4oPhrg38i7
Rh4c+3GrfJO9SkvzcEy5uPrX3p5a8uEYW7Kp0vyZhubjmCLx1egFEVfJi2Oh
735Sc+hhDc3LscyspIFV5ZfJT3i/8U193qFqml/CuHRH2sdpavKUsNCjEfri
jZdoHxKm3Xd31RNcJF/h+bq6t+dvvED7Eb5/kazbk1pF3hJmPbmoO/HT87Qv
KZNVLTydUHqO/KWMO3ugt6OpkvYnZV3H8V2hpYLOg5RtU8Tvvbm8gvYpZQaV
r1TV9QudDymryCxcmba3nPYbxAwhOenB/p/p/w1imfNCMzzqMvwL2wMqhQ==

         "]]}, {}}}, {}, {}, {}, {}}, {
    DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {0, 0}}, AxesOrigin -> {0.978125, 0}, 
     PlotRange -> {{1., 2.4}, {0, 1}}, PlotRangeClipping -> True, 
     ImagePadding -> All, DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0.978125, 0}, DisplayFunction :> 
     Identity, Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"Rescue probability\"", TraditionalForm], None}, {
        FormBox[
        "\"Antibiotic concentration \\!\\(\\*SubscriptBox[\\(c\\), \\(out\\)]\
\\)\"", TraditionalForm], None}}, FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {{0}, {0}}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], LabelStyle -> {FontFamily -> "Times"}, 
     Method -> {"CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, PlotLabel -> 
     FormBox["\"Absolute - Panel E\"", TraditionalForm], 
     PlotRange -> {{1., 2.4}, {0, 1}}, PlotRangeClipping -> True, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {0, 0}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"1", "2", "5", "10"}, "PointLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               StyleBox[
               "\"Plasmid copy \\nnumber \
\\!\\(\\*StyleBox[\\\"n\\\",FontSlant->\\\"Italic\\\"]\\)\"", {
                FontFamily -> "Times", FontFamily -> "Arial"}, Background -> 
                Automatic, StripOnInput -> False]}, {
               TagBox[
                GridBox[{{
                   TagBox[
                    GridBox[{{
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.9, 0.36, 0.054], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.9, 0.36, 0.054], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.365248, 0.427802, 0.758297], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.365248, 0.427802, 0.758297], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.945109, 0.593901, 0.], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.945109, 0.593901, 0.], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.645957, 0.253192, 0.685109], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.645957, 0.253192, 0.685109], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                 False, GridBoxItemSize -> {
                  "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                 GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
                "Grid"]}}, GridBoxAlignment -> {"Columns" -> {{Center}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
           Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Times", FontFamily -> "Arial"}, Background -> 
         Automatic, StripOnInput -> False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"PointLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.9, 0.36, 0.054], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.6000000000000001, 0.24, 0.036000000000000004`],
                     FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.9`", ",", "0.36`", ",", "0.054`"}], "]"}], 
                    NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.9, 0.36, 0.054]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.9, 0.36, 0.054], Editable -> False, Selectable -> 
                    False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.365248, 0.427802, 0.758297], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.2434986666666667, 0.28520133333333336`, 
                    0.5055313333333333], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.365248`", ",", "0.427802`", ",", "0.758297`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.365248, 0.427802, 0.758297]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.365248, 0.427802, 0.758297], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.945109, 0.593901, 0.], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0.6300726666666667, 0.395934, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.945109`", ",", "0.593901`", ",", "0.`"}], 
                    "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.945109, 0.593901, 0.]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.945109, 0.593901, 0.], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.645957, 0.253192, 0.685109], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.430638, 0.16879466666666665`, 0.45673933333333333`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.645957`", ",", "0.253192`", ",", "0.685109`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.645957, 0.253192, 0.685109]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.645957, 0.253192, 0.685109], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
           RowBox[{"Joined", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{"False", ",", "False", ",", "False", ",", "False"}], 
               "}"}]}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{"FontFamily", "\[Rule]", "\"Times\""}], "}"}]}], ",", 
           
           RowBox[{"LegendLabel", "\[Rule]", 
             "\"Plasmid copy \\nnumber \
\\!\\(\\*StyleBox[\\\"n\\\",FontSlant->\\\"Italic\\\"]\\)\""}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.7857454304224453`*^9, 3.785745841252161*^9, 
  3.785863028462064*^9},
 CellLabel->"Out[56]=",ExpressionUUID->"298174f5-ad16-452a-a8b2-6d9c757afe73"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"type", "=", "\"\<AbsoluteCost - Panel F\>\""}], ";"}], 
  "\n"}], "\n", 
 RowBox[{
  RowBox[{"t", "=", 
   RowBox[{"CalculateRescueProbabilities", "[", "type", "]"}]}], 
  ";"}], "\n", 
 RowBox[{"GridData", "[", "t", "]"}], "\n", 
 RowBox[{"ListPlotData", "[", 
  RowBox[{"t", ",", "type"}], "]"}], "\n"}], "Code",
 InitializationCell->False,
 CellChangeTimes->{{3.785741055921051*^9, 3.785741058494282*^9}, 
   3.785742197771072*^9},
 CellLabel->"In[57]:=",ExpressionUUID->"07597354-b90e-4cdd-8114-318f22790cb0"],

Cell[BoxData[
 TagBox[GridBox[{
    {"\<\"\"\>", "1.`", "1.16`", "1.32`", "1.48`", "1.6400000000000001`", 
     "1.8`", "1.96`", "2.12`", "2.2800000000000002`"},
    {"1", "0.913060502753358`", "0.0308713395228164`", "0.`", "0.`", "0.`", 
     "0.`", "0.`", "0.`", "0.`"},
    {"2", "0.8607545276743116`", "0.17129756347844904`", 
     "0.027047207980190557`", "0.`", "0.`", "0.`", "0.`", "0.`", "0.`"},
    {"5", "0.7181503681126787`", "0.23553172453916438`", 
     "0.11303833796086216`", "0.0598466778957093`", "0.02986865104249381`", 
     "0.009043308466653577`", "0.`", "0.`", "0.`"},
    {"10", "0.539884269295268`", "0.2051661252560415`", 
     "0.09987008241224615`", "0.054447757626997495`", "0.03175545271626157`", 
     "0.019284534282714705`", "0.011934261016117054`", 
     "0.007348948859877247`", "0.004327855314503415`"}
   },
   AutoDelete->False,
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output",
 CellChangeTimes->{
  3.785744528387833*^9, 3.785744645266968*^9, 3.785744760779211*^9, 
   3.785744827083928*^9, 3.785744928263262*^9, 3.7857450529169197`*^9, 
   3.785745106874853*^9, 3.7857451760743113`*^9, {3.7857452218810596`*^9, 
   3.785745251544477*^9}, 3.785745304032546*^9, 3.785745391574329*^9, 
   3.785745444797374*^9, 3.785745893598723*^9, 3.785863102098242*^9},
 CellLabel->"Out[59]=",ExpressionUUID->"badb67d0-115a-49a5-af0e-0d38f6db2644"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{}, {{{}, {
        Hue[0.67, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.9, 0.36, 0.054], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJxd0z9IQlEUx/GXDYEEQTgXNVUQFEVNdtxsVcQhnIIaIuq1RBiBi0FTLQWK
pFCQkzVEOLhkU5GDNEhDkNLUH/BZ2h//9d49dH90L4h8lsM996sD86veBZum
aW7zY33zKdOI/S55M/1G3nRf+MVTpqVE/DR5UaJj3WkvHJXpoHs5OthzT7Wh
wG62WqapDcN1GMnTbDHoSLkN8j105Z7GrykasY5BY+cZX/j5kl491kCDEv7e
k/x+mmbMabqzQo9zV7F4K0XmMHNihbSVne1MLE5iXLFCodDZ6JZ/nSbEeae/
m5qXMwfC1jR7AR62rjf0Ib1pjQvCuVvrwP1iYFV6zVpXh8X1srAY56hJL4oD
i+dLw2Jd+6e0uF4AFuNSMJ8vae4Bcw+Ye8Dc41uae8DcA+YeP9J7YmG4JHrA
k6JHXelRV3rUlR4NpUdD6dFQejSlxbp6U+nRVHq0lB4tpUdL6dFWerSVHm2l
h+b6vw/MfWDeD+ZeMO8Lcz+Y9++Q5p4wvwfMfWF+H5h7w/xeNmnuD/P7wfx7
gPk9Yf59wPy+ndL8/4V/ARXGN4M=
         "]]}, {
        Hue[0.9060679774997897, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.365248, 0.427802, 0.758297], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJxd001IVFEYxvGb7WbV4hIuoiYymBaRUNvxUVyMBC2cVdBsjLIWYTeIItsI
oggi1tKUMBraDAhFwiCUMBQllJBQE0KkYmBYMF/X/KiZ7j0PnYfugWH4bV7O
ef8zRy9eT19uchwnFXzCb54Sloop99zoJtL5w0Ob3SWsuTufq9/XkfWSseLj
Et68WHz7qnMFW4nMeMEv4Xlut6/RvIyu1X53JlXGwaH5zEPvIx5MhKcMd+Dr
o+WND/jRHQ4sY3sueepYxyLagmlesoLEyO3hT/kFBMOCiRV0xQcPuT9fw4xb
reCa6758kivgtDlVjJydPX8pO4/gcsHAKuIHxvta788hnBYrVtF+vDl3cm8W
J8LrJWq4ujE1nF54irvhuP4aSoM9d6aWcnj/Ljw1xFs7bl6YzOKIGehjume9
t/PZJG6Ez/V8DLTdavnmj8Fcr+DDWZke9b5cgRnnbuHf5nrNkc368rJ5buyX
tbleRjbjZmSebWv2kNlDZg+ZPXas2UNmD5k9dq3vmQfLa6aHfMb02LNmD5k9
ZPb4bc0eMnvI7PHH2jzXk9lDZo96pEc90qMe6dGI9GhEejQiPZz2/98js4/M
98nsJfO9MvvJfP8+a/aUuQ+ZfWXuR2ZvmftqsmZ/mfuT+XuQuU+Zvw+Z+91v
zf+v/BfrAFxW
         "]]}, {
        Hue[0.1421359549995791, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.945109, 0.593901, 0.], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJxFk29IU1EYxq/2xfxQhkOFaC1JcJB/WhKEzGdEpljSlBRTQcySyKCrEomD
UkHsg9KkDDUN0SUxxlgma5iQjjSjlGHUpKj8UxYkNTc3p5vWvfccd1647P4Y
PJzn/Z176OL1/MvhHMdlCY/4S8YFpUumiwksI98mb/6d54LPuffaftUSDLw6
0jngQmt5XHypZh6+xNK7dq8Lbs5qKV7+jOyFepk5axXG22XaIfUcurvEWUVq
lbKgOusDVvLEwFW4mw6nRDtnkSGk8Wo3uPTOsgPTDghhQqIb/rqIcz8TZiDF
LbhheVzYGJP6Fsek8UAzcFOr6JuCcDgh0AP9paSovIpJiGmRTg+4nC+x6qpX
UIrHS1zDsMOq6DbZoRPj6tew0psZXy0fx/Q7cdZQtNRbE1v7EgelQC9c/GiP
zDWKarEu78WdsLbkJOMIpOPZvbCpFk6d77JBipP54EiZ1k1araiUxocrK8nt
U8FhSOuzCZxzIfdG5TNIdSPXkai1vnjw9ymk45WuY8rmn2u5b4EUZ16H46gp
+4TBDOLDD36PVS4/Y6I+/IhqTCr5FmekPgT+vtWj2v2E+vDDcss4nhs3SH1s
QN9mz/l40kB9bCDqj2rwfXM/9bEBhbPu9cDXPupjE32zPeqljkfQS4U3MV+X
kVDc/hCLko9NaNK95RGmTqRJPgJQzE88H1nsoD4CGDP76w2qe9RHAFxu0VBh
l576CEKjnyjgla3URxBcoL0hmNlCfQQxlvZDe7a5ifrYAhe97+rIGx2kuvwW
Gmo/zZyeqKE+hP/L+4/4PRXUxza4X3aeLzhOfWxj52YTH4yJj38hJj4YEx+M
6Wh2XkgfxsQPY9KPMfHFmPRlTPwxJv3DQkx8Mib7YEz8Mib7YUx8Myb7Cg8x
8c+Y7I8xuQ+MyT4Zk/vBmOx3V4jJ98v4P9fouYE=
         "]]}, {
        Hue[0.37820393249936934`, 0.6, 0.6], 
        Directive[
         PointSize[0.007333333333333334], 
         RGBColor[0.645957, 0.253192, 0.685109], 
         CapForm["Butt"], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJwtkw1MjHEcx58LoZnyktWGOzYmaRp5yarv4y1sJtVthTahJK93QyUvhZBr
dKlOJbleGFo6Rq/iFCsRea15vaJ5p7u6Xu6u43+P32979uyzZ/vu//19/s+k
jbuCIu04jlvGHtv7/3Ti05+h4dX+bQiqmHjsR2AnTFcSFrZ/eY8ima9DS2En
ltyKCxnl+gY908JSa42dmPEYsTGG11jeFj/22jI9soIyR7YGvEROtm30aP8z
L6LM+Tl+BtoC9Tjg6zXdb2Yz/FiazNcAy6u148SuTWBhLNGArxvGz25f1Agh
rs0AySr70sqGeswWpgtS1f01+ecegB2OBXYhdF3Jy7zSOtjSHFq68NkaVjDM
fA9utuNN64bmbQFv3avFfltcfDfCz2cr7b3uoOmxbbrhNCU8NaquGmIh0Ijl
c5/qndMqIbfVlRkRB9GvnZnlEI5Xa4RkaXGH4eEtCHFje5C1+G2Lk9tNbBam
B/O3ZQ4ZKLkBYX0VjJtcZiml1yHUdeiFZ2P5Jg9XDYTjhfVCo7ktXaMvgRB3
rReSHTcy0nXF+O+jD7qkFx3u5ivkow/q0KKqFp/L5IN9rz+keZR/iXz0gYsx
a69OvUg++qGV95piGwrJRz+UY06vViQVkI9+6GoqxUdC88mHCZKiBx78IjWU
QmET+Pc55dUJeWgXfJigk+Y2rryQCy/BhxnatFmDJ+tyyIcZfPYHVSyyyYcZ
ah/fM0+qzpIPCyQnPcUdASryYYHu8PGyUksG+WC8jvcMvptOPgagLpSeFqvO
QKgrY7ytOTolIY18DCDcv+znzHgl+bCCM/ekKIankg8rEhO8HeFzinxYoc2z
qw1RpJCPv0isLAx+pz9JPhiPmbR5QJZMPv6C/xZS7GR/gnxwPNdq2XGq9Bj1
YVwfvW/49iTyw/Ha3D7p3AVHqR/H80r3hx9cjpAv9v22x6XP9oepL+O81ff9
BiWSP5b3bo7st+sh6i/iufWqLcH+B8gn49Fat8GKeNqHiE/0Vk+I+RRHfhlf
8BYtCIyl/TD+XqHtf7aXfDMulifHROyhfdnxXF3UkBNDd5N/xo45zc5Vctof
4xF1nGS/jO4D402RNVHpO2mfjN0V8o8rttP9YPz8fJHIZSvtdxDP7VvcusQS
Rf8v44Mu9d7GSPwDqPYZJw==
         "]]}, {}}}, {}, {}, {}, {}}, {
    DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {0, 0}}, AxesOrigin -> {0.978125, 0}, 
     PlotRange -> {{1., 2.4}, {0, 1}}, PlotRangeClipping -> True, 
     ImagePadding -> All, DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0.978125, 0}, DisplayFunction :> 
     Identity, Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"Rescue probability\"", TraditionalForm], None}, {
        FormBox[
        "\"Antibiotic concentration \\!\\(\\*SubscriptBox[\\(c\\), \\(out\\)]\
\\)\"", TraditionalForm], None}}, FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {{0}, {0}}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], LabelStyle -> {FontFamily -> "Times"}, 
     Method -> {"CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, PlotLabel -> 
     FormBox["\"AbsoluteCost - Panel F\"", TraditionalForm], 
     PlotRange -> {{1., 2.4}, {0, 1}}, PlotRangeClipping -> True, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {0, 0}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"1", "2", "5", "10"}, "PointLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               StyleBox[
               "\"Plasmid copy \\nnumber \
\\!\\(\\*StyleBox[\\\"n\\\",FontSlant->\\\"Italic\\\"]\\)\"", {
                FontFamily -> "Times", FontFamily -> "Arial"}, Background -> 
                Automatic, StripOnInput -> False]}, {
               TagBox[
                GridBox[{{
                   TagBox[
                    GridBox[{{
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.9, 0.36, 0.054], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.9, 0.36, 0.054], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.365248, 0.427802, 0.758297], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.365248, 0.427802, 0.758297], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.945109, 0.593901, 0.], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.945109, 0.593901, 0.], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.645957, 0.253192, 0.685109], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.264], 
                    RGBColor[0.645957, 0.253192, 0.685109], 
                    CapForm["Butt"], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                 False, 
                 GridBoxItemSize -> {
                  "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                 GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
                "Grid"]}}, GridBoxAlignment -> {"Columns" -> {{Center}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
           Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Times", FontFamily -> "Arial"}, Background -> 
         Automatic, StripOnInput -> False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"PointLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.9, 0.36, 0.054], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.6000000000000001, 0.24, 0.036000000000000004`],
                     FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.9`", ",", "0.36`", ",", "0.054`"}], "]"}], 
                    NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.9, 0.36, 0.054]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.9, 0.36, 0.054], Editable -> False, Selectable -> 
                    False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.365248, 0.427802, 0.758297], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.2434986666666667, 0.28520133333333336`, 
                    0.5055313333333333], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.365248`", ",", "0.427802`", ",", "0.758297`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.365248, 0.427802, 0.758297]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.365248, 0.427802, 0.758297], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.945109, 0.593901, 0.], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0.6300726666666667, 0.395934, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.945109`", ",", "0.593901`", ",", "0.`"}], 
                    "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.945109, 0.593901, 0.]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.945109, 0.593901, 0.], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.007333333333333334`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.645957, 0.253192, 0.685109], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.430638, 0.16879466666666665`, 0.45673933333333333`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.645957`", ",", "0.253192`", ",", "0.685109`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.645957, 0.253192, 0.685109]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.645957, 0.253192, 0.685109], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"CapForm", "[", "\"Butt\"", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
           RowBox[{"Joined", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{"False", ",", "False", ",", "False", ",", "False"}], 
               "}"}]}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{"FontFamily", "\[Rule]", "\"Times\""}], "}"}]}], ",", 
           
           RowBox[{"LegendLabel", "\[Rule]", 
             "\"Plasmid copy \\nnumber \
\\!\\(\\*StyleBox[\\\"n\\\",FontSlant->\\\"Italic\\\"]\\)\""}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.785744528387833*^9, 3.785744645266968*^9, 3.785744760779211*^9, 
   3.785744827083928*^9, 3.785744928263262*^9, 3.7857450529169197`*^9, 
   3.785745106874853*^9, 3.7857451760743113`*^9, {3.7857452218810596`*^9, 
   3.785745251544477*^9}, 3.785745304032546*^9, 3.785745391574329*^9, 
   3.785745444797374*^9, 3.785745893598723*^9, 3.785863102345584*^9},
 CellLabel->"Out[60]=",ExpressionUUID->"8d261b85-5f12-4efe-b822-e96a61419de4"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{909, 755},
WindowMargins->{{0, Automatic}, {20, Automatic}},
FrontEndVersion->"11.3 for Mac OS X x86 (32-bit, 64-bit Kernel) (March 5, \
2018)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 726, 20, 90, "Section",ExpressionUUID->"a42136b8-3aa0-47fb-a4f4-51df235afb8c"],
Cell[CellGroupData[{
Cell[1331, 46, 208, 4, 54, "Subsection",ExpressionUUID->"570af557-ecad-40f8-91fd-5e4fb83ea7fd",
 InitializationCell->True],
Cell[1542, 52, 1469, 30, 131, "Input",ExpressionUUID->"90316cf1-c581-4446-b4e6-b7d11d92456e",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[3048, 87, 189, 4, 54, "Subsection",ExpressionUUID->"4de71919-c89c-4900-ae99-6017d699bbf4",
 InitializationCell->True],
Cell[3240, 93, 3956, 96, 338, "Code",ExpressionUUID->"32ae2f82-96cf-4734-84db-a3546d4b5524"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7233, 194, 178, 4, 54, "Subsection",ExpressionUUID->"656f169a-d280-4184-90ec-b30a49761ee6",
 InitializationCell->True],
Cell[7414, 200, 4566, 114, 208, "Code",ExpressionUUID->"e4d41e4c-d6b8-450f-8506-eafbbcfadc91"],
Cell[11983, 316, 3324, 82, 204, "Code",ExpressionUUID->"88bddaee-4787-42ec-8c44-eb4e8b11dc00"],
Cell[15310, 400, 1689, 40, 91, "Code",ExpressionUUID->"975d7ee5-23ba-4b3c-b689-6c4d2d66b5d9"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17036, 445, 257, 6, 54, "Subsection",ExpressionUUID->"15f85401-bfcf-46cb-b0fc-7a4c4795f265",
 InitializationCell->True],
Cell[17296, 453, 9257, 232, 614, "Input",ExpressionUUID->"fd1f9ca5-eb57-40a0-9bd2-aa6127eccde8",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[26590, 690, 1456, 37, 216, "Subsection",ExpressionUUID->"3da18ec3-fc87-4e5f-9d77-52b6def5cf92"],
Cell[28049, 729, 3791, 94, 217, "Input",ExpressionUUID->"49699357-0200-4822-8eb8-c49da288e8a6",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[31865, 827, 1488, 27, 113, "Code",ExpressionUUID->"337e31d1-b7bc-428f-b8f2-6a7c7d0ef4c9",
 InitializationCell->False],
Cell[33356, 856, 1346, 23, 99, "Output",ExpressionUUID->"999f6abb-2002-4bf2-97da-43e0d9dcaf25"],
Cell[34705, 881, 26686, 543, 269, "Output",ExpressionUUID->"7b87aef2-584f-4c17-a24a-2642f5ae8895"]
}, Open  ]],
Cell[CellGroupData[{
Cell[61428, 1429, 602, 14, 132, "Code",ExpressionUUID->"99634223-9886-40e3-99ee-4948c143dfa6",
 InitializationCell->False],
Cell[62033, 1445, 1071, 20, 99, "Output",ExpressionUUID->"79668249-8c14-4943-b1dd-c63117e37147"],
Cell[63107, 1467, 25557, 524, 269, "Output",ExpressionUUID->"298174f5-ad16-452a-a8b2-6d9c757afe73"]
}, Open  ]],
Cell[CellGroupData[{
Cell[88701, 1996, 561, 15, 132, "Code",ExpressionUUID->"07597354-b90e-4cdd-8114-318f22790cb0",
 InitializationCell->False],
Cell[89265, 2013, 1419, 25, 99, "Output",ExpressionUUID->"badb67d0-115a-49a5-af0e-0d38f6db2644"],
Cell[90687, 2040, 25779, 527, 269, "Output",ExpressionUUID->"8d261b85-5f12-4efe-b822-e96a61419de4"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

